@@ -26,10 +26,10 @@ use Type::Tiny::Role;
 package Attean::API::Plan 0.009 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed);
-	use Types::Standard qw(ArrayRef CodeRef Str Object InstanceOf Bool Num);
+	use Types::Standard qw(ArrayRef CodeRef Str Object InstanceOf Bool Num Int);
 	use namespace::clean;
 	
-	has 'cost' => (is => 'rw', isa => Num, predicate => 'has_cost');
+	has 'cost' => (is => 'rw', isa => Int, predicate => 'has_cost');
 	has 'distinct' => (is => 'rw', isa => Bool, required => 1, default => 0);
 	has 'item_type' => (is => 'ro', isa => Str, required => 1, default => 'Attean::API::Result');
 	has 'in_scope_variables' => (is => 'ro', isa => ArrayRef[Str], required => 1);
