@@ -127,7 +127,7 @@ package Attean::API::SimpleCostPlanner 0.009 {
 		my $self	= shift;
 		my $plan	= shift;
 		my $model	= shift;
-		Carp::confess unless ref($model);
+		Carp::confess "No model given" unless ref($model);
 		
 		if ($plan->has_cost) {
 			return $plan->cost;
@@ -323,7 +323,7 @@ package Attean::API::IDPJoinPlanner 0.009 {
 		my $self	= shift;
 		my $plan	= shift;
 		my $model	= shift;
-		Carp::confess unless ref($model);
+		Carp::confess "No model given" unless ref($model);
 		
 		if ($plan->has_cost) {
 			return $plan->cost;
@@ -385,6 +385,8 @@ package Attean::API::IDPJoinPlanner 0.009 {
 				}
 			}
 			
+			# Costs must be integers for comparisons to work in the IDP planning algorithm
+			$cost	= int($cost);
 			$plan->cost($cost);
 			return $cost;
 		}
